#ifndef __field_h__
#define __field_h__

/* #define STRANGE_CONSTANT 1.0L / 1.643933566682L */
#define STRANGE_CONSTANT 100.0

extern int num_queued_fields;

/* waver velosity */
extern double waver_velosity;

/* strange constant */
extern double strange_constant;

#define CIRCS 360 

typedef struct field {
  struct qelem q;
  double origin_x,origin_y;
  double current_radius;
  double mass;
  int field_number;
  int particle_number;
  int del;
  struct field *pair; /* if this field is paired, point to it */
  int new_flag;
  int flag;           /* copy from point                      */
} FIELD, *FIELD_PTR;

/* number of queued fields */
extern int num_queued_fields;

#ifdef __cplusplus
extern "C" {
#endif

/* get rid of new flag */
void no_new ( void );

/* optomize number of fields */
int opt_fields ( void );

/* create and queue up a new field */
int new_field ( double x, double y, double mass, int particle_number, int flag, double radius );

/* remove from queue and destroy a field */
void destroy_field ( FIELD_PTR f );

/* tick a field */
int tick_a_field ( FIELD_PTR f, double delta_time );

/* tick all fields */
int tick_all_fields ( double delta_time );

/* get_one_force
 *
 * m     - mass, either +1 or -1
 * [x,y] - location of m
 * [i,j] - my current location
 *
 * Returns: TRUE if in range, else FALSE
 */
int get_one_force ( int m, int x, int y, int i, int j, double *r_dx, double *r_dy );

/* get forces at x,y from all fields */
int get_all_forces ( POINT_PTR p, double *fx, double *fy, int trace_flag );

#ifdef __cplusplus
};
#endif

#endif /* __field_h__ */
