/* myidle.c */

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <search.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "qelem.h"
#include "points.h"
#include "manifold.h"
#include "myidle.h"
#include "scribble-simple.h"
#include "field.h"
#include "sym.h"

/* overall trace flag */
int trace_flag = 0;

/* state engine state */
gint idle_state = 0;

/* current simulation time */
gint current_time_cnt = 0;
double current_time = 0.0;

/* number of seconds for each baby step in simulation */
#define SYM_INC 0.001
double sym_inc = SYM_INC;

/* show aspect */
extern void show_aspect();

#define LINE_Y (WIN_SIZE_Y/2 -1)
#define SLIT_SIZE 10
int slit_distance = 20;
/* WIN_SIZE_X */

int grid_flag = 0;

/* draw some grid lines */
void draw_grid ( )
{
  int x,y;

  if ( !grid_flag ) return;

#define LININC 10

  for ( y = 0 ; y < 640 ; y += LININC ) {
    gdk_draw_line ( pixmap,
		    drawing_area->style->black_gc,
		    0,y,
		    640,y );
  }

#if 1
  for ( x = 0 ; x < 640 ; x += LININC ) {
    gdk_draw_line ( pixmap,
		    drawing_area->style->black_gc,
		    x,0,
		    x, 640);
  }
#endif

}

#if 0
/* draw two slits */
void draw_two_slits ( int my_y )
{
  int x1, x2;
  int y1 = my_y, y2 = my_y;

  x1 = 0;
  x2 = WIN_SIZE_X / 2 - slit_distance / 2 - SLIT_SIZE;

  /* left to first slit */
  gdk_draw_line ( pixmap,
		  drawing_area->style->black_gc,
		  x1,y1,
		  x2,y2 );

  /* skip first slit */
  x1 = x2 + SLIT_SIZE;
  x2 = slit_distance / 2 + WIN_SIZE_X / 2;

  /* draw middle */
  gdk_draw_line ( pixmap,
		  drawing_area->style->black_gc,
		  x1,y1,
		  x2,y2 );

  x1 = x2 + SLIT_SIZE;
  x2 = WIN_SIZE_X;

  /* draw right */
  gdk_draw_line ( pixmap,
		  drawing_area->style->black_gc,
		  x1,y1,
		  x2,y2 );
}
#endif

#if 0
/* draw circs */
void draw_circs ( int x, int y, int r )
{
  int left_x = x - r;
  int top_y  = y - r;
  int wid = 2 * r;
  int hid = 2 * r;

  gdk_draw_arc (
		pixmap,
		drawing_area->style->black_gc,
		FALSE,
		left_x, top_y,
		wid, hid,
		0,
		64 * 360/2 );
}
#endif

/* draw circs */
void draw_circs ( int x, int y, int r )
{
  int left_x = x - r;
  int top_y  = y - r;
  int wid = 2 * r;
  int hid = 2 * r;

  gdk_draw_arc (
		pixmap,
		drawing_area->style->black_gc,
		FALSE,
		left_x, top_y,
		wid, hid,
		0,
		64 * 360 );
}

/* draw uncircs */
void draw_uncircs ( int x, int y, int r )
{
  int left_x = x - r;
  int top_y  = y - r;
  int wid = 2 * r;
  int hid = 2 * r;

  gdk_draw_arc (
		pixmap,
		drawing_area->style->black_gc,
		FALSE,
		left_x, top_y,
		wid, hid,
		0,
		64 * 360 );
}

#if 1
/* draw elipse */
void draw_elipse ( int x, int y, int r, double div )
{
  int left_x = x - r;
  int top_y  = y - ((double)r)*div;
  int wid = 2 * r;
  int hid = 2 * ((double)r)*div;

  gdk_draw_arc (
		pixmap,
		drawing_area->style->black_gc,
		FALSE,
		left_x, top_y,
		wid, hid,
		0,
		64 * 360/2 );
}
#endif

/* undraw elipse */
void undraw_elipse ()
{
  gdk_draw_rectangle (pixmap,
		      drawing_area->style->white_gc,
		      TRUE,
		      0   /* x */      ,
		      0   /* y */      ,
		      WIN_SIZE_X /* width  */ ,
		      WIN_SIZE_Y /* height */
		      );
}

int run_flag       =0;
double cur_div     = 1.0;
double cur_div_inc = -0.001;

/* inc */
void do_inc()
{
  cur_div += cur_div_inc;
}

/* dec */
void do_dec()
{
  cur_div -= cur_div_inc;
}

int circ_flag = 0;

extern GtkWidget *window;

/*
 * 0 - no step
 * 1 - step forward
 * -1 - step back
 */

int step_flag = 0;

/* draw an arc */
int draw_arc ( )
{
  int x;
  int r;
  static xinc = 50;
  int rcnt;

  undraw_elipse();

  draw_grid();

  x = xinc;

  for ( r = 50 ; r < WIN_SIZE_Y ; r += 11 ) {
    /* draw circs */
    draw_circs ( 
		/* x */ x,
		/* y */ LINE_Y,
		/* radius */ r );
  }

  x = WIN_SIZE_X / 2 + 60;

  for ( r = 50 ; r < WIN_SIZE_Y ; r += 11 ) {
    /* draw circs */
    draw_circs ( 
		/* x */ x,
		/* y */ LINE_Y,
		/* radius */ r );
  }

  //g_printf("drawing arcs, %lf\n",cur_div);

 if ( 1 == step_flag ) {
   if ( xinc < WIN_SIZE_X - 50 ) {
     xinc++;
   }
 } else {
   if ( -1 == step_flag ) {
     if ( xinc > 50 ) {
       xinc--;
     }
   }
 }

 if ( xinc >= WIN_SIZE_X ) {
   run_flag = 0;
   xinc = 50;
 } else {
   if ( run_flag ) {
     xinc++;
     usleep(100000);
   }
 }

  gtk_widget_queue_draw_area (drawing_area, 
			      0,0,
			      WIN_SIZE_X, WIN_SIZE_Y);

  return run_flag ? 1 : 0;
}

gint nap ()
{
  return 0;
}

/* Use gtk idle time for simulation,
 * return FALSE if we're done else
 * TRUE
 */
gint my_idle ( gpointer data )
{
  gint sts = 1;

  /* off to appropriate state */
  switch ( idle_state )
    {
    case 0:
      sts = draw_arc();
      break;

    case 1:
      sts = draw_arc();
      break;

    }

  /* return to caller */
  return sts;
}
