/** points.c **/

#include <stdio.h>
#include <stdlib.h>
#include <search.h>
#include <assert.h>
#include <string.h>
#include <math.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gdk/gdk.h>

#include "qelem.h"
#include "manifold.h"
#include "points.h"
#include "field.h"

#include "scribble-simple.h"

#if 0
/* display text here */
extern GtkWidget *text;
/* describe a font */
extern GdkFont *fixed_font;
/* Backing pixmap for drawing area */
extern GdkPixmap *pixmap;
/* drawing widget */
extern GtkWidget *drawing_area;
#endif

/* a constant used with 'C' calcuations */
double n_const = 0.01; /* ??? Why this value ??? */

/*
 * define a group of points
 */
int pts_idx = 0;
POINT pts[POINT_MAX];

/* #define CP_TRACE */

/*
 * Move a point 'tim' seconds
 *
 * Returns: TRUE if stays in bounds,
 *          else FALSE
 */
int mover_once ( POINT_PTR p, double tim )
{
  double delta_x  , delta_y  ;
  double delta_v_x, delta_v_y;
  int    pi, pj;
  POINT  p_initial;
  int    ipi,ipj;
  double field_x,field_y;

#ifdef CP_TRACE1
  printf("mover_once: entry\n");
#endif

  /* save initial conditions    */
  /* calculate manifold indexes */
  memmove(&p_initial,p,sizeof(POINT));
  pi = ipi = p->x;
  pj = ipj = p->y;

  /* get delta x and y */
  delta_x = p->v_x * tim;
  delta_y = p->v_y * tim;

  if ( delta_x > 1.0 || delta_y > 1.0 ) {
    printf("mover_once::warning: big delta x,y = %f %f\n",
	   delta_x,
	   delta_y);
  }

  /* is point still in range ??? */
  if ( pi >= WIN_SIZE_X || pj >= WIN_SIZE_Y || pi < 0 || pj < 0 ) {

#ifdef CP_TRACE
    /* no */
    printf("mover_once: not still in range, returning 0, pi = %d, pj = %d\n",
	   pi,pj);
#endif

    /* return done to caller */
    return 0;
  }

  /* get forces at point 'p' from all fields */
  get_all_forces ( p, &field_x, &field_y, 0);

#if 0
  printf("mover_once: p(%d) at [%3d,%3d], f's = %f %f, man = %f %f, net %f %f\n",
	 p->particle_number,
	 pi,pj,
	 field_x,field_y,
	 man[pi][pj].x,man[pi][pj].y,
	 field_x + man[pi][pj].x, field_y + man[pi][pj].y 
	 );
#endif

  /* get total forces */
  field_x += man[pi][pj].x;
  field_y += man[pi][pj].y;

  if ( 4 & p->flag ) {
    double n_delta_v_x;
    double n_delta_v_y;
    /* double n_mag; */
    double rel_x,rel_y;

#if 1

    /* consider 'C' in delta 'V' calculation
     * based on:
     *
     * delta v = K * (C/(C-V0)) * sqrt ( f * delta_t / m )
     */

    /* n_mag = sqrt ( p->v_x * p->v_x + p->v_y * p->v_y ); */

    rel_x = (waver_velosity - fabs(p->v_x))/waver_velosity;
    rel_x *= rel_x;
    rel_y = (waver_velosity - fabs(p->v_y))/waver_velosity;
    rel_y *= rel_y;
    n_delta_v_x = fabs(n_const * rel_x * field_x * tim / p->mass);
    n_delta_v_y = fabs(n_const * rel_y * field_y * tim / p->mass);

#if 0
    printf("vx = %f -> %f, vy = %f -> %f\n",
	   p->v_x,rel_x,
	   p->v_y,rel_y);
#endif

#else
    /* consider 'C' in delta 'V' calculation
     * based on:
     *
     *  delta v = K * sqrt ( f * delta_t / (C - V0) * m )
     */

    n_mag = sqrt ( p->v_x * p->v_x + p->v_y * p->v_y );
    n_delta_v_x = n_const * sqrt ( fabs(field_x * tim / ( waver_velosity - n_mag ) * p->mass) );
    n_delta_v_y = n_const * sqrt ( fabs(field_y * tim / ( waver_velosity - n_mag ) * p->mass) );
#endif

    /* play with the sign */
    if ( field_x < 0.0 ) {
      n_delta_v_x *= -1;
    }
    if ( field_y < 0.0 ) {
      n_delta_v_y *= -1;
    }

#if 0
    /* display results for debugging */
    printf("New [%2.16f, %2.16f], Old [%f, %f], k = %f %f\n",
	   n_delta_v_x,n_delta_v_y,
	   field_x * tim, field_y * tim,
	   fabs( field_x * tim / n_delta_v_x ),
	   fabs( field_y * tim / n_delta_v_y )
	   );
#endif

    delta_v_x = n_delta_v_x / p->mass;
    delta_v_y = n_delta_v_y / p->mass;

  } else {

    /* get delta velosity */
    delta_v_x = field_x * tim / p->mass;
    delta_v_y = field_y * tim / p->mass;

  }

  /* update position and acceleration */
  p->x   += delta_x;
  p->y   += delta_y;
  p->v_x += delta_v_x;
  p->v_y += delta_v_y;

  /* are we still in range ??? */
  if ( p->x >= WIN_SIZE_X || p->y >= WIN_SIZE_Y ||
      p->x < 0 || p->y < 0 ) {
    return 0;
  }

  /* if we are in a new cell, we need to send out
   * a wave
   */
  pi = p->x;
  pj = p->y;
  if ( pi != ipi || pj != ipj ) {
    /* something changed */
    double radius = sqrt(8.0);

    /* launch a negative mass field from the old position */
    new_field ( p_initial.x, p_initial.y, -1.0*p->mass, p->particle_number, p->flag, radius );

    /* launch a positive mass field from the new position */
    new_field ( p->x       , p->y       , p->mass     , p->particle_number, p->flag, radius );
  }

  /* return to caller */
  return 1;
}

/* show points velosity */
void show_points_vel ( void )
{
  int i;
  char wbuf[256];

  for ( i = 0 ; i < pts_idx ; i++ ) {
    POINT_PTR p;

    p = &pts[i];

    sprintf(wbuf,"Point p%d, [vx,vy] = [%f,%f], mag |%f|\n",
	    i,
	    p->v_x, p->v_y,
	    sqrt ( p->v_x * p->v_x + p->v_y * p->v_y )
	    );
    gtk_text_insert (GTK_TEXT(text), fixed_font, NULL, NULL,wbuf, -1 );
  }
}

/* draw all points */
void draw_all_points ( void )
{
  int i;

  for ( i = 0 ; i < pts_idx ; i++ ) {

    GdkRectangle update_rect;

    update_rect.x = pts[i].x - POINT_SIZE;
    update_rect.y = pts[i].y - POINT_SIZE;
    update_rect.width = POINT_SIZE;
    update_rect.height = POINT_SIZE;
    gdk_draw_rectangle (pixmap,
			drawing_area->style->black_gc,
			TRUE,
			update_rect.x, update_rect.y,
			update_rect.width, update_rect.height);
    gtk_widget_queue_draw_area (drawing_area, 
				update_rect.x, update_rect.y,
				update_rect.width, update_rect.height);

  }

#if 0
  /* first point */
  update_rect.x = p1.x - POINT_SIZE;
  update_rect.y = p1.y - POINT_SIZE;
  update_rect.width = POINT_SIZE;
  update_rect.height = POINT_SIZE;
  gdk_draw_rectangle (pixmap,
		      drawing_area->style->black_gc,
		      TRUE,
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
  gtk_widget_queue_draw_area (drawing_area, 
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);

  /* second point */
  update_rect.x = p2.x - POINT_SIZE;
  update_rect.y = p2.y - POINT_SIZE;
  update_rect.width = POINT_SIZE;
  update_rect.height = POINT_SIZE;
  gdk_draw_rectangle (pixmap,
		      drawing_area->style->black_gc,
		      TRUE,
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
  gtk_widget_queue_draw_area (drawing_area, 
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
#endif

#if 0
  gdk_draw_string (pixmap,
		   fixed_font,
		   drawing_area->style->black_gc,
		   100,100,
		   "The quick brown fox JUMPED OVER THE LAZY DOG" );

  gtk_widget_queue_draw_area (drawing_area, 
		      90,90,
		      200,50);
#endif
}
